@@ -3,4 +3,5 @@ from enum import Enum
 
 class SearchEngine(Enum):
     DUCKDUCKGO = 1,
-    STARTPAGE = 2
+    STARTPAGE = 2,
+    BING = 3
