@@ -1,8 +1,9 @@
 import json
 import time
+import urllib
 from multiprocessing import Process
 from time import sleep
-from urllib.parse import urlparse, urlencode, parse_qsl
+from urllib.parse import urlparse, parse_qsl
 
 from selenium.common.exceptions import TimeoutException, WebDriverException, MoveTargetOutOfBoundsException
 from selenium.webdriver import ActionChains
@@ -87,6 +88,7 @@ def get_auth_resp(chromedriver, tuple_url_and_counter):
     parsed_auth_req = urlparse(auth_req.url)
     counter_req = tuple_url_and_counter[1]
     counter = 0
+    has_redirect_req = 'redirect_uri' in auth_req.params
     postmessage_req = parsed_auth_req.netloc == "malicious.com"
     storagerelay = parsed_auth_req.query.lower().__contains__("redirect_uri=storagerelay")
     staticxx_facebook = 'redirect_uri' in auth_req.params and auth_req.params['redirect_uri'].startswith(
@@ -100,13 +102,14 @@ def get_auth_resp(chromedriver, tuple_url_and_counter):
             counter += 1
             continue
         postmessage_res = parsed_req.netloc == "malicious.com"
+        has_location_header_res = request.response is not None and 'location' in request.response.headers
+        fragment_mock_request = request.url.startswith("https://pocs.work")
         if postmessage_req and postmessage_res:
-            json_obj = json.loads(request.body.decode('utf8'))
-            auth_req_json_obj = json.loads(auth_req.body.decode("utf8"))
             try:
+                json_obj = json.loads(request.body.decode('utf8'))
+                auth_req_json_obj = json.loads(auth_req.body.decode("utf8"))
                 if json_obj["origin"] == auth_req_json_obj["documentLocation"]["origin"] and "type" in json_obj[
-                    "data"] and \
-                        json_obj["data"]["type"].lower() == "response":
+                    "data"] and json_obj["data"]["type"].lower() == "response":
                     return request
             except Exception:
                 # Nothing to do --> wrong message format
@@ -138,32 +141,24 @@ def get_auth_resp(chromedriver, tuple_url_and_counter):
             except Exception:
                 # Nothing to do --> wrong message format
                 pass
-        else:
-            if request.url.startswith(auth_req.params['redirect_uri']):
-                u = request.url
-                if u.__contains__("code") or u.__contains__("access_token") or u.__contains__("id_token"):
+        elif has_redirect_req:
+            u = request.url
+            decoded_body = None
+            if fragment_mock_request:
+                frag_url = urllib.parse.unquote(request.params["oreq"])
+                if ProcessHelper.find_redirect_uri_in_request(frag_url, request.params["oreq"]):
                     return request
-                if form_post_request and len(request.body) > 0:
-                    try:
-                        decoded_body = request.body.decode("utf-8")
-                        if decoded_body.__contains__("code=") or decoded_body.__contains__(
-                                "access_token=") or decoded_body.__contains__("id_token="):
-                            return request
-                    except Exception:
-                        # Nothing to do --> wrong message format
-                        pass
-
-            try:
-                # Fallback if redirect uri did contain unencoded url characters
-                parsed = urlparse(auth_req.params['redirect_uri'])
-                parsed = parsed._replace(query=urlencode(parse_qsl(parsed.query)))
-                parse_url = parsed.geturl()
-                if request.url.startswith(parse_url):
-                    if parse_url.__contains__("code") or parse_url.__contains__(
-                            "access_token") or parse_url.__contains__("id_token"):
-                        return request
-            except Exception:
-                pass
+            if form_post_request and len(request.body) > 0:
+                try:
+                    decoded_body = request.body.decode("utf-8")
+                except Exception:
+                    # Nothing to do --> wrong message format
+                    pass
+            if ProcessHelper.find_redirect_uri_in_request(u, auth_req.params['redirect_uri'], decoded_body):
+                return request
+            if has_location_header_res and ProcessHelper.find_redirect_uri_in_request(
+                    request.response.headers['location'], auth_req.params['redirect_uri']):
+                return request
 
 
 def check_login_successfully(chromedriver, social_login_info_to_find):
@@ -190,6 +185,12 @@ def check_login_successfully(chromedriver, social_login_info_to_find):
             return_res = auth_res.body.decode("utf-8")
         except Exception:
             return_res = auth_res.url
+    elif auth_req is not None and 'location' in auth_req[0].response.headers and (
+            auth_req[0].response.headers['location'].__contains__("code") or auth_req[0].response.headers[
+        'location'].__contains__("access_token") or auth_req[0].response.headers['location'].__contains__("id_token")):
+        return_res = auth_req[0].response.headers['location']
+    elif auth_res.url.startswith("https://pocs.work"):
+        return_res = auth_res.params['oreq']
     else:
         return_res = auth_res.url
     return True, return_req, return_res
@@ -268,19 +269,25 @@ def process_function(sso_detection, backend_info: BackendInformation, analysis_r
                                                                     "Pressing element coordinates")
         logger.info("Pressing coordinates")
         action = ActionChains(chromedriver)
-        action.move_by_offset(element_x, element_y).click().move_by_offset(element_x * -1, element_y * -1).perform()
-        logger.info("Waiting for login finished")
-        rest_client.update_progress_of_analysis_using_sso_detection(analysis_run_id, sso_detection_index, 50,
-                                                                    "Waiting for finished login after pressing element coordinates")
-        sleep(1)
-        result = wait_for_result(chromedriver, social_login_info)
-        if result[0]:
-            success = handle_result_found(result, rest_client, analysis_run_id, sso_detection_index, screen,
-                                          chromedriver.har)
-            return
-
+        try:
+            action.move_by_offset(element_x, element_y).click().move_by_offset(element_x * -1, element_y * -1).perform()
+            logger.info("Waiting for login finished")
+            rest_client.update_progress_of_analysis_using_sso_detection(analysis_run_id, sso_detection_index, 50,
+                                                                        "Waiting for finished login after pressing element coordinates")
+            sleep(1)
+            result = wait_for_result(chromedriver, social_login_info)
+            if result[0]:
+                success = handle_result_found(result, rest_client, analysis_run_id, sso_detection_index, screen,
+                                              chromedriver.har)
+                return
+        except MoveTargetOutOfBoundsException:
+            logger.warn(
+                "Looks like original coordinates are no longer valid for the current login page. Will use fallback")
+            element_width = None
+            element_height = None
+        error_screen = chromedriver.get_screenshot_as_png()
         if element_width is not None and element_height is not None:
-            logger.info("Pressing at coordinates did not get results. Trying to press in center of element")
+            logger.info("Pressing at coordinates did not get a valid result. Trying to press in center of element")
             rest_client.update_progress_of_analysis_using_sso_detection(analysis_run_id, sso_detection_index, 55,
                                                                         "Pressing element's center")
             chromedriver.get(finding_url)
@@ -293,9 +300,10 @@ def process_function(sso_detection, backend_info: BackendInformation, analysis_r
             rest_client.update_progress_of_analysis_using_sso_detection(analysis_run_id, sso_detection_index, 50,
                                                                         "Waiting for finished login after pressing element's center")
             result = wait_for_result(chromedriver, social_login_info)
-            success = handle_result_found(result, rest_client, analysis_run_id, sso_detection_index, screen,
-                                          chromedriver.har)
-            return
+            if result[0]:
+                success = handle_result_found(result, rest_client, analysis_run_id, sso_detection_index, screen,
+                                              chromedriver.har)
+                return
         if sso_detection_strategy == SSODetectionStrategy.ELEMENT_SEARCH:
             from processes import SSODetectionService
             from model.login_path_information import LoginPathInformation
@@ -329,7 +337,8 @@ def process_function(sso_detection, backend_info: BackendInformation, analysis_r
                 rest_client.update_progress_of_analysis_using_sso_detection(analysis_run_id, sso_detection_index, 90,
                                                                             "Uploading failed analysis data")
                 rest_client.save_sso_security_analysis(analysis_run_id, sso_detection_index, (result[1], result[2]),
-                                                       screen, chromedriver.har, False, "Did not find auth req and res")
+                                                       screen, chromedriver.har, False, "Did not find auth req and res",
+                                                       error_screenshot=error_screen)
             else:
                 logger.info("Fallback located element successfully, saving results")
                 rest_client.update_progress_of_analysis_using_sso_detection(analysis_run_id, sso_detection_index, 90,
